import pytest
import numpy
from gym.spaces import Box
from numpy import array


def test_attributes(three_way_daily_env):
    assert three_way_daily_env.action_space == Box(low=0, high=2 ** 3 - 0.01, shape=(2,))
    assert three_way_daily_env.observation_space == Box(low=0., high=float('Inf'), shape=(2, 3))
    assert three_way_daily_env.STARTING_BANK == 10
    assert three_way_daily_env.balance == three_way_daily_env.STARTING_BANK
    assert three_way_daily_env.current_step == 0
    assert numpy.array_equal(three_way_daily_env.bet_size_matrix, numpy.ones(shape=(2, 3)))
    assert numpy.array_equal(three_way_daily_env.teams.values, numpy.array([['FCB', 'PSG'], ['MCB', 'MTA'],
                                                                            ['PSG', 'MCB'], ['FCB', 'MTA'],
                                                                            ['PSG', 'FCB'], ['MTA', 'MCB']]))


@pytest.mark.parametrize("action,expected_reward", [((0, 0), 0), ((1, 1), 2), ((2, 2), 0), ((3, 3), -2), ((4, 4), 2),
                                                    ((5, 5), 0), ((6, 6), -2), ((7, 7), 0)])
def test_step(three_way_daily_env, action, expected_reward):
    odds, reward, done, _ = three_way_daily_env.step(action)
    assert reward == expected_reward
    assert not done
    assert three_way_daily_env.current_step == 1


@pytest.mark.parametrize("current_step,action,expected_reward", [(0, array([0, 0]), 0), (0, array([5, 4, 7]), 0),
                                                                 (0, array([2, 7]), 2), (1, array([1, 7]), -1),
                                                                 (1, array([1]), -1), (1, array([3, 6, 7]), 2),
                                                                 (2, array([2]), 2), (2, array([2, 2]), 1),
                                                                 (2, array([2, 1, 3]), 3), (2, array([6, 0, 7]), 0)])
def test_step_non_uniform(three_way_daily_env_non_uniform, current_step, action, expected_reward):
    three_way_daily_env_non_uniform.current_step = current_step
    odds, reward, done, _ = three_way_daily_env_non_uniform.step(action)
    assert reward == expected_reward


def test_multiple_steps(three_way_daily_env):
    odds, reward, done, _ = three_way_daily_env.step((3, 3))
    assert reward == -2
    assert three_way_daily_env.balance == three_way_daily_env.STARTING_BANK - 2
    assert not done
    assert three_way_daily_env.current_step == 1
    odds, reward, done, _ = three_way_daily_env.step((1, 1))
    assert reward == -2
    assert three_way_daily_env.balance == three_way_daily_env.STARTING_BANK - 2 - 2
    assert not done
    assert three_way_daily_env.current_step == 2
    odds, reward, done, _ = three_way_daily_env.step((2, 2))
    assert reward == -2
    assert three_way_daily_env.balance == three_way_daily_env.STARTING_BANK - 2 - 2 - 2
    assert done


def test_multiple_steps_non_uniform(three_way_daily_env_non_uniform):
    current_bank = three_way_daily_env_non_uniform.STARTING_BANK
    odds, reward, done, _ = three_way_daily_env_non_uniform.step(array([2, 7]))
    assert three_way_daily_env_non_uniform.balance == current_bank + 2
    assert not done
    assert three_way_daily_env_non_uniform.current_step == 1
    current_bank += 2
    odds, reward, done, _ = three_way_daily_env_non_uniform.step(array([3, 6, 7]))
    assert three_way_daily_env_non_uniform.balance == current_bank + 2
    assert not done
    assert three_way_daily_env_non_uniform.current_step == 2
    current_bank += 2
    odds, reward, done, _ = three_way_daily_env_non_uniform.step(array([2, 1, 3]))
    assert three_way_daily_env_non_uniform.balance == current_bank + 3
    assert done



import numpy
from gym import spaces
from .base import BaseOddsEnv


class BasePercentageOddsEnv(BaseOddsEnv):
    """Base class for sports betting environments with non fixed bet size.

    Creates an OpenAI Gym environment that supports betting a non fixed amount
    on a single outcome for a single game.

    The main difference between the BaseOddsEnv is that the action space is defined
    differently (to accommodate that non fixed bet size).

    .. versionadded:: 0.2.0
    .. deprecated:: 0.4.0
        This environment will be renamed "BasePercentageOddsEnv" in 0.4.5
    .. versionchanged:: 0.4.5
        Name changed to "BasePercentageOddsEnv"
    .. versionchanged:: 0.5.0
        Change action space so that each outcome has it's own independent bet
        percentage.

    Parameters
    ----------
    action_space : gym.spaces.Box of shape (N+1,), where N is the number of possible
        outcomes for the game.
        The (N+1)-tuple first index is the action itself and the rest of the
        indexes are the percentage of the current balance to place on matching
        outcome, so that action[i + 1] is the bet percentage for outcome[i].

    """

    def __init__(self, odds, odds_column_names, results=None):
        super().__init__(odds, odds_column_names, results)
        self.bet_size_matrix = None
        self.action_space = spaces.Box(low=numpy.array([0.] * (odds.shape[1] + 1)),
                                       high=numpy.array([self.action_space.n - 0.01] + [1.] * odds.shape[1]))

    def step(self, action):
        form = int(numpy.floor(action[0]))
        self.bet_size_matrix = numpy.array(action[1:]) * self.balance
        return super().step(form)



from .base import BaseOddsEnv
from .base_percentage import BasePercentageOddsEnv
from .daily_bets import DailyOddsEnv, DailyPercentageOddsEnv


class ThreeWaySoccerOddsEnv(BaseOddsEnv):
    """Environment for 3-way soccer betting with a fixed bet size.

    .. versionadded:: 0.1.0
    """

    def __init__(self, soccer_bets_dataframe):
        """Initializes a new environment.

        Parameters
        ----------
        soccer_bets_dataframe: dataframe of shape (n_games, n_odds + 3), n_odds == 3
            A list of games, with their betting odds, results and the teams playing.

            .. warning::

                Please make sure that the odds columns are named "home, draw, away",
                that the results column is named "result" and that the team names
                columns are named "home_team, away_team" respectively.
        """
        odds_column_names = ['home', 'draw', 'away']
        odds = soccer_bets_dataframe[odds_column_names].values
        results = soccer_bets_dataframe['result'].values if soccer_bets_dataframe['result'].notna().all() else None
        super().__init__(odds, odds_column_names, results)
        self.teams = soccer_bets_dataframe[['home_team', 'away_team']].values

    def render(self, mode='human'):  # pragma: no cover
        """Outputs the current team names, balance and step.

        Returns
        -------
        msg : str
            A string with the current team names, balance and step.
        """
        return 'Home Team {} VS Away Team {}. {}'.format(self.teams[self.current_step % self._odds.shape[0]][0],
                                                         self.teams[self.current_step % self._odds.shape[0]][1],
                                                         super().render(mode))


class ThreeWaySoccerPercentageOddsEnv(ThreeWaySoccerOddsEnv, BasePercentageOddsEnv):
    """Environment for 3-way soccer betting with a non fixed bet size.

    .. versionadded:: 0.2.0
    """
    pass


class ThreeWaySoccerDailyOddsEnv(DailyOddsEnv):
    """Environment for 3-way soccer betting, grouped by day, with a fixed bet size.

    .. versionadded:: 0.5.0
    """

    def __init__(self, soccer_bets_dataframe, *args, **kwargs):
        """Initializes a new environment.



        Parameters
        ----------
        soccer_bets_dataframe: dataframe of shape (n_games, n_odds + 4), n_odds == 3
            A list of games, with their betting odds, results, the teams playing and the dates.

            .. warning::

                Please make sure that the odds columns are named "home, draw, away",
                that the results column is named "result", that the team names
                columns are named "home_team, away_team" respectively and that the
                date column is named "date".
        """
        odds_column_names = ['home', 'draw', 'away']
        odds = soccer_bets_dataframe[odds_column_names + ['date']]
        results = soccer_bets_dataframe['result'] if soccer_bets_dataframe['result'].notna().all() else None
        super().__init__(odds, odds_column_names, results, *args, **kwargs)
        self.teams = soccer_bets_dataframe[['home_team', 'away_team']]

    def render(self, mode='human'):  # pragma: no cover
        """Outputs the current team names, balance and step.

        Returns
        -------
        msg : str
            A string with the current team names, balance and step.
        """
        index = self._get_current_index()
        teams_str = ', '.join(['Home Team {} VS Away Team {}'.format(row.home_team, row.away_team)
                               for row in self.teams.iloc[index].itertuples()])
        balance_str = super().render(mode)
        return '. '.join((teams_str, balance_str))


class ThreeWaySoccerDailyPercentageOddsEnv(ThreeWaySoccerDailyOddsEnv, DailyPercentageOddsEnv):
    """Environment for 3-way soccer betting, grouped by date, with a non fixed bet size.

    .. versionadded:: 0.5.0
    """
    pass



import pytest
import numpy
from gym.spaces import Box


def test_attributes(basic_percentage_env):
    assert basic_percentage_env.action_space == Box(low=numpy.array([0.] * 3),
                                                    high=numpy.array([2 ** 2 - 0.01] + [1.] * 2))
    assert basic_percentage_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 2))
    assert basic_percentage_env.STARTING_BANK == 10
    assert basic_percentage_env.balance == basic_percentage_env.STARTING_BANK
    assert basic_percentage_env.current_step == 0
    assert basic_percentage_env.bet_size_matrix is None


@pytest.mark.parametrize("action,expected_reward", [((0, 0.25, 0.25), 0), ((1, 0.25, 0.25), 2.5),
                                                    ((2, 0.25, 0.25), -2.5), ((3, 0.25, 0.25), 0)])
def test_step(basic_percentage_env, action, expected_reward):
    odds, reward, done, _ = basic_percentage_env.step(action)
    assert reward == expected_reward
    assert done
    assert basic_percentage_env.current_step == 0



import pytest
import numpy
from gym.spaces import Box


def test_attributes(three_way_percentage_env):
    assert three_way_percentage_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 3))
    assert three_way_percentage_env.action_space == Box(low=numpy.array([0.] * 4),
                                                        high=numpy.array([2 ** 3 - 0.01] + [1.] * 3))
    assert numpy.array_equal(three_way_percentage_env.teams, numpy.array([['FCB', 'PSG'], ['MCB', 'MTA']]))


@pytest.mark.parametrize("action,expected_reward", [((0, 0.1, 0.1, 0.1), 0),
                                                    ((1, 0.1, 0.1, 0.1), -1),
                                                    ((2, 0.1, 0.1, 0.1), 1),
                                                    ((3, 0.1, 0.1, 0.1), -1)])
def test_step(three_way_percentage_env, action, expected_reward):
    odds, reward, done, _ = three_way_percentage_env.step(action)
    assert reward == expected_reward
    assert not done
    assert three_way_percentage_env.current_step == 1



import pytest
import numpy
from gym.spaces import Box
from numpy import array


def test_attributes(three_way_daily_percentage_env):
    assert three_way_daily_percentage_env.action_space == Box(low=array([[0] * 4] * 2).reshape(8),
                                                              high=array([[2 ** 3 - 0.01] + [1] * 3] * 2).reshape(8))
    assert three_way_daily_percentage_env.observation_space == Box(low=0., high=float('Inf'), shape=(2, 3))
    assert three_way_daily_percentage_env.STARTING_BANK == 10
    assert three_way_daily_percentage_env.balance == three_way_daily_percentage_env.STARTING_BANK
    assert three_way_daily_percentage_env.current_step == 0
    assert numpy.array_equal(three_way_daily_percentage_env.bet_size_matrix, numpy.ones(shape=(2, 3)))
    assert numpy.array_equal(three_way_daily_percentage_env.teams.values, array([['FCB', 'PSG'], ['MCB', 'MTA'],
                                                                                 ['PSG', 'MCB'], ['FCB', 'MTA'],
                                                                                 ['PSG', 'FCB'], ['MTA', 'MCB']]))


@pytest.mark.parametrize("action,expected_reward", [(array([[0] * 4] * 2).reshape(8), 0),
                                                    (array([[1] + [0.1] * 3] * 2).reshape(8), 2),
                                                    (array([[2] + [0.1] * 3] * 2).reshape(8), 0),
                                                    (array([[3] + [0.1] * 3] * 2).reshape(8), -2),
                                                    (array([[4] + [0.1] * 3] * 2).reshape(8), 2),
                                                    (array([[5] + [0.1] * 3] * 2).reshape(8), 0),
                                                    (array([[6] + [0.1] * 3] * 2).reshape(8), -2),
                                                    (array([[7] + [0.1] * 3] * 2).reshape(8), 0)])
def test_step(three_way_daily_percentage_env, action, expected_reward):
    odds, reward, done, _ = three_way_daily_percentage_env.step(action)
    assert reward == expected_reward
    assert not done
    assert three_way_daily_percentage_env.current_step == 1


@pytest.mark.parametrize("current_step,action,expected_reward",
                         [(0, array([0, 0.1, 0.1, 0.1, 0, 0.1, 0.1, 0.1]), 0),
                          (0, array([0, 0.1, 0.1, 0.1, 0, 0.1, 0.1, 0.1, 0, 0.1, 0.1, 0.1]), 0),
                          (0, array([2, 0.0, 0.25, 0.0, 7, 0.2, 0.1, 0.1]), 6.5),
                          # Reward = (0.25*10*2)+(0.2*10*4)-0.25*10-0.2*10-0.1*10-0.1*10 = 6.5
                          (1, array([6, 0.0, 0.0, 0.0]), 0),
                          (1, array([6, 0.0, 0.4, 0.5]), 6),
                          # Reward = (0.5*10*3)-0.4*10-0.5*10 = 6
                          (2, array([2, 0.0, 0.15, 0.0, 1, 0.135, 0.135, 0.1, 7, 0.2, 0.2, 0.2]), 1),
                          # Reward = (0.15*10*3)+(0.135*10*1)+(0.2*10*2)-0.15*10-0.135*10-3*(0.2*10) = 1
                          (2, array([2, 0.0, 0.15, 0.0, 1, 0.135, 0.135, 0.1, 7, 0.4, 0.2, 0.2]), -10.85)
                          # Illegal bet
                          ])
def test_step_non_uniform(three_way_daily_percentage_env_non_uniform, current_step, action, expected_reward):
    three_way_daily_percentage_env_non_uniform.current_step = current_step
    odds, reward, done, _ = three_way_daily_percentage_env_non_uniform.step(action)
    assert reward == expected_reward


def test_multiple_steps(three_way_daily_percentage_env):
    odds, reward, done, _ = three_way_daily_percentage_env.step(array([[3] + [0.1] * 3] * 2).reshape(8))
    assert reward == -2
    assert three_way_daily_percentage_env.balance == three_way_daily_percentage_env.STARTING_BANK - 2
    assert not done
    assert three_way_daily_percentage_env.current_step == 1
    bet_matrix = [1 / three_way_daily_percentage_env.balance] * 3
    odds, reward, done, _ = three_way_daily_percentage_env.step(array([[1] + bet_matrix] * 2).reshape(8))
    assert reward == -2
    assert three_way_daily_percentage_env.balance == three_way_daily_percentage_env.STARTING_BANK - 2 - 2
    assert not done
    assert three_way_daily_percentage_env.current_step == 2
    bet_matrix = [1 / three_way_daily_percentage_env.balance] * 3
    odds, reward, done, _ = three_way_daily_percentage_env.step(array([[2] + bet_matrix] * 2).reshape(8))
    assert reward == -2
    assert three_way_daily_percentage_env.balance == three_way_daily_percentage_env.STARTING_BANK - 2 - 2 - 2
    assert done


def test_multiple_steps_non_uniform(three_way_daily_percentage_env_non_uniform):
    current_bank = three_way_daily_percentage_env_non_uniform.STARTING_BANK
    odds, reward, done, _ = three_way_daily_percentage_env_non_uniform.step(array([2, 0.0, 0.25, 0.0,
                                                                                   7, 0.2, 0.1, 0.1]))
    assert reward == 6.5
    assert three_way_daily_percentage_env_non_uniform.balance == current_bank + 6.5
    assert not done
    assert three_way_daily_percentage_env_non_uniform.current_step == 1
    current_bank += reward
    odds, reward, done, _ = three_way_daily_percentage_env_non_uniform.step(array([2, 0.0, 0.4, 0.0]))
    assert reward == -current_bank * 0.4
    assert three_way_daily_percentage_env_non_uniform.balance == current_bank - current_bank * 0.4
    assert not done
    assert three_way_daily_percentage_env_non_uniform.current_step == 2
    current_bank += reward
    odds, reward, done, _ = three_way_daily_percentage_env_non_uniform.step(array([2, 0.0, 0.15, 0.0,
                                                                                   1, 0.135, 0.135, 0.1,
                                                                                   7, 0.2, 0.2, 0.2]))
    assert round(reward, 2) == 0.99
    assert done



"""This module contains the packaging routine for oddsgym"""
from os import path
from setuptools import setup, find_packages
from configparser import ConfigParser


requirements = {'install_requires': ['gym', 'more-itertools', 'pandas']}
requirements['extra_requires'] = {'examples': requirements['install_requires'] + ['tensorflow<2', 'stable-baselines']}


def _get_version():
    with open(path.join(path.dirname(__file__), 'setup.cfg')) as setup_configuration_file:
        parser = ConfigParser()
        parser.read_string(setup_configuration_file.read())
        return parser['bumpversion']['current_version']


setup(packages=find_packages(),
      install_requires=requirements['install_requires'],
      extra_requires=requirements['extra_requires'],
      version=_get_version(),
      python_requires=">=3.6")



import pytest
from gym.spaces import Box
from pandas import DataFrame
from datetime import datetime, timedelta
import numpy
from numpy import array, zeros
from oddsgym.envs.daily_bets import DailyPercentageOddsEnv



@pytest.mark.parametrize("max_number_of_games", ['auto', 1])
def test_max_number_of_games_valid(max_number_of_games):
    dataframe = DataFrame([{'w': 4, 'l': 3, 'date': datetime.today().date(), 'result': 0}])
    DailyPercentageOddsEnv(dataframe.drop('result', 'columns'), ['w', 'l'], dataframe['result'], max_number_of_games)


@pytest.mark.parametrize("max_number_of_games", [0, None, 3.5, 'some random string'])
def test_max_number_of_games_invalid(max_number_of_games):
    dataframe = DataFrame([{'w': 4, 'l': 3, 'date': datetime.today().date(), 'result': 0}])
    with pytest.raises(ValueError):
        DailyPercentageOddsEnv(dataframe.drop('result', 'columns'), ['w', 'l'], dataframe['result'],
                               max_number_of_games)


def test_attributes(daily_bets_percentage_env):
    assert daily_bets_percentage_env.action_space == Box(low=zeros(shape=(6,)),
                                                         high=array([2 ** 2 - 0.01, 1, 1] * 2).reshape(6))
    assert daily_bets_percentage_env.observation_space == Box(low=0., high=float('Inf'), shape=(2, 2))
    assert daily_bets_percentage_env.STARTING_BANK == 10
    assert daily_bets_percentage_env.balance == daily_bets_percentage_env.STARTING_BANK
    assert daily_bets_percentage_env.current_step == 0
    assert len(daily_bets_percentage_env.days) == 2
    assert daily_bets_percentage_env.days[0] == datetime.today().date() - timedelta(days=1)
    assert daily_bets_percentage_env.days[1] == datetime.today().date()


def test_get_odds(daily_bets_percentage_env):
    odds = daily_bets_percentage_env.get_odds()
    assert odds.equals(DataFrame([{'w': 1, 'l': 2}, {'w': 4, 'l': 3}], dtype=numpy.float64))


@pytest.mark.parametrize("actions,bets", [(array([0, 0]), array([[0, 0], [0, 0]])),
                                          (array([0, 1]), array([[0, 0], [1, 0]])),
                                          (array([0, 2]), array([[0, 0], [0, 1]])),
                                          (array([0, 3]), array([[0, 0], [1, 1]])),
                                          (array([1, 0]), array([[1, 0], [0, 0]])),
                                          (array([1, 1]), array([[1, 0], [1, 0]])),
                                          (array([1, 2]), array([[1, 0], [0, 1]])),
                                          (array([1, 3]), array([[1, 0], [1, 1]])),
                                          (array([2, 0]), array([[0, 1], [0, 0]])),
                                          (array([2, 1]), array([[0, 1], [1, 0]])),
                                          (array([2, 2]), array([[0, 1], [0, 1]])),
                                          (array([2, 3]), array([[0, 1], [1, 1]])),
                                          (array([3, 0]), array([[1, 1], [0, 0]])),
                                          (array([3, 1]), array([[1, 1], [1, 0]])),
                                          (array([3, 2]), array([[1, 1], [0, 1]])),
                                          (array([3, 3]), array([[1, 1], [1, 1]])),
                                          ])
def test_get_bet(daily_bets_percentage_env, actions, bets):
    assert numpy.array_equal(daily_bets_percentage_env.get_bet(actions), bets)


@pytest.mark.parametrize("current_step,action,expected_reward,finished",
                         [(0, array([[0, 0.1, 0.1], [0, 0.1, 0.1]]).reshape(6), 0, False),
                          (1, array([[0, 0.1, 0.1], [0, 0.1, 0.1]]).reshape(6), 0, True),
                          (0, array([[1, 0.1, 0.1], [1, 0.1, 0.1]]).reshape(6), 2, False),
                          (1, array([[1, 0.1, 0.1], [1, 0.1, 0.1]]).reshape(6), 6, True),
                          (0, array([[1, 0.1, 0.1], [2, 0.1, 0.1]]).reshape(6), -2, False),
                          (1, array([[1, 0.1, 0.1], [2, 0.1, 0.1]]).reshape(6), 2, True),
                          (0, array([[2, 0.1, 0.1], [2, 0.1, 0.1]]).reshape(6), 0, False),
                          (1, array([[2, 0.1, 0.1], [2, 0.1, 0.1]]).reshape(6), -2, True),
                          (0, array([[3, 0.1, 0.1], [3, 0.1, 0.1]]).reshape(6), 2, False),
                          (1, array([[3, 0.1, 0.1], [3, 0.1, 0.1]]).reshape(6), 4, True),
                          (0, array([[0, 0., 0.], [0, 0., 0.]]).reshape(6), 0, False),
                          (1, array([[0, 0., 0.], [0, 0., 0.]]).reshape(6), 0, True),
                          (0, array([[1, 0., 0.], [1, 0., 0.]]).reshape(6), 0, False),
                          (1, array([[1, 0., 0.], [1, 0., 0.]]).reshape(6), 0, True),
                          (0, array([[1, 0., 0.], [2, 0., 0.]]).reshape(6), 0, False),
                          (1, array([[1, 0., 0.], [2, 0., 0.]]).reshape(6), 0, True),
                          (0, array([[2, 0., 0.], [2, 0., 0.]]).reshape(6), 0, False),
                          (1, array([[2, 0., 0.], [2, 0., 0.]]).reshape(6), 0, True),
                          (0, array([[3, 0., 0.], [3, 0., 0.]]).reshape(6), 0, False),
                          (1, array([[3, 0., 0.], [3, 0., 0.]]).reshape(6), 0, True)])
def test_step(daily_bets_percentage_env, current_step, action, expected_reward, finished):
    daily_bets_percentage_env.current_step = current_step
    odds, reward, done, info = daily_bets_percentage_env.step(action)
    assert reward == expected_reward
    assert done == finished


def test_attributes_of_non_uniform(daily_bets_percentage_env_non_uniform):
    assert daily_bets_percentage_env_non_uniform.action_space == Box(low=zeros(shape=(9)),
                                                                     high=array([2 ** 2 - 0.01, 1., 1.] * 3).reshape(9))
    assert daily_bets_percentage_env_non_uniform.observation_space == Box(low=0., high=float('Inf'), shape=(3, 2))
    assert len(daily_bets_percentage_env_non_uniform.days) == 3
    assert daily_bets_percentage_env_non_uniform.days[0] == datetime.today().date() - timedelta(days=2)
    assert daily_bets_percentage_env_non_uniform.days[1] == datetime.today().date() - timedelta(days=1)
    assert daily_bets_percentage_env_non_uniform.days[2] == datetime.today().date()


@pytest.mark.parametrize("current_step,excpected_odds",
                         [(0, DataFrame([{'w': 1, 'l': 2}, {'w': 0, 'l': 0}, {'w': 0, 'l': 0}],
                                        dtype=numpy.float64)),
                          (1, DataFrame([{'w': 4, 'l': 3}, {'w': 0, 'l': 0}, {'w': 0, 'l': 0}],
                                        dtype=numpy.float64)),
                          (2, DataFrame([{'w': 4, 'l': 3}, {'w': 4, 'l': 3}, {'w': 5, 'l': 4}],
                                        dtype=numpy.float64))])
def test_get_odds_non_uniform(daily_bets_percentage_env_non_uniform, current_step, excpected_odds):
    daily_bets_percentage_env_non_uniform.current_step = current_step
    odds = daily_bets_percentage_env_non_uniform.get_odds()
    assert odds.equals(excpected_odds)


@pytest.mark.parametrize("actions,bets", [(array([0]), zeros([3, 2])),
                                          (array([0] * 2), zeros([3, 2])),
                                          (array([0] * 3), zeros([3, 2])),
                                          (array([1]), array([[1, 0], [0, 0], [0, 0]])),
                                          (array([1] * 2), array([[1, 0], [1, 0], [0, 0]])),
                                          (array([1] * 3), array([[1, 0], [1, 0], [1, 0]])),
                                          (array([2]), array([[0, 1], [0, 0], [0, 0]])),
                                          (array([2] * 2), array([[0, 1], [0, 1], [0, 0]])),
                                          (array([2] * 3), array([[0, 1], [0, 1], [0, 1]])),
                                          (array([3]), array([[1, 1], [0, 0], [0, 0]])),
                                          (array([3] * 2), array([[1, 1], [1, 1], [0, 0]])),
                                          (array([3] * 3), array([[1, 1], [1, 1], [1, 1]])),
                                          (array([0, 1]), array([[0, 0], [1, 0], [0, 0]])),
                                          (array([0, 2]), array([[0, 0], [0, 1], [0, 0]])),
                                          (array([0, 3]), array([[0, 0], [1, 1], [0, 0]])),
                                          (array([1, 0]), array([[1, 0], [0, 0], [0, 0]])),
                                          (array([1, 2]), array([[1, 0], [0, 1], [0, 0]])),
                                          (array([1, 3]), array([[1, 0], [1, 1], [0, 0]])),
                                          (array([2, 0]), array([[0, 1], [0, 0], [0, 0]])),
                                          (array([2, 1]), array([[0, 1], [1, 0], [0, 0]])),
                                          (array([2, 3]), array([[0, 1], [1, 1], [0, 0]])),
                                          (array([3, 0]), array([[1, 1], [0, 0], [0, 0]])),
                                          (array([3, 1]), array([[1, 1], [1, 0], [0, 0]])),
                                          (array([3, 2]), array([[1, 1], [0, 1], [0, 0]])),
                                          (array([0, 0, 1]), array([[0, 0], [0, 0], [1, 0]])),
                                          (array([0, 0, 2]), array([[0, 0], [0, 0], [0, 1]])),
                                          (array([0, 0, 3]), array([[0, 0], [0, 0], [1, 1]])),
                                          (array([0, 1, 0]), array([[0, 0], [1, 0], [0, 0]])),
                                          (array([0, 2, 0]), array([[0, 0], [0, 1], [0, 0]])),
                                          (array([0, 3, 0]), array([[0, 0], [1, 1], [0, 0]])),
                                          (array([1, 0, 0]), array([[1, 0], [0, 0], [0, 0]])),
                                          (array([2, 0, 0]), array([[0, 1], [0, 0], [0, 0]])),
                                          (array([3, 0, 0]), array([[1, 1], [0, 0], [0, 0]])),
                                          (array([0, 1, 1]), array([[0, 0], [1, 0], [1, 0]])),
                                          (array([0, 1, 2]), array([[0, 0], [1, 0], [0, 1]])),
                                          (array([0, 1, 3]), array([[0, 0], [1, 0], [1, 1]])),
                                          (array([0, 2, 1]), array([[0, 0], [0, 1], [1, 0]])),
                                          (array([0, 2, 2]), array([[0, 0], [0, 1], [0, 1]])),
                                          (array([0, 2, 3]), array([[0, 0], [0, 1], [1, 1]])),
                                          (array([0, 3, 1]), array([[0, 0], [1, 1], [1, 0]])),
                                          (array([0, 3, 2]), array([[0, 0], [1, 1], [0, 1]])),
                                          (array([0, 3, 3]), array([[0, 0], [1, 1], [1, 1]])),
                                          (array([1, 0, 1]), array([[1, 0], [0, 0], [1, 0]])),
                                          (array([1, 0, 2]), array([[1, 0], [0, 0], [0, 1]])),
                                          (array([1, 0, 3]), array([[1, 0], [0, 0], [1, 1]])),
                                          (array([1, 1, 0]), array([[1, 0], [1, 0], [0, 0]])),
                                          (array([1, 1, 2]), array([[1, 0], [1, 0], [0, 1]])),
                                          (array([1, 1, 3]), array([[1, 0], [1, 0], [1, 1]])),
                                          (array([1, 2, 0]), array([[1, 0], [0, 1], [0, 0]])),
                                          (array([1, 2, 1]), array([[1, 0], [0, 1], [1, 0]])),
                                          (array([1, 2, 2]), array([[1, 0], [0, 1], [0, 1]])),
                                          (array([1, 2, 3]), array([[1, 0], [0, 1], [1, 1]])),
                                          ])
def test_get_bet_non_uniform(daily_bets_percentage_env_non_uniform, actions, bets):
    assert numpy.array_equal(daily_bets_percentage_env_non_uniform.get_bet(actions), bets)


@pytest.mark.parametrize("current_step,expected_results",
                         [(0, array([[0, 1], [0, 0], [0, 0]], dtype=numpy.float64)),
                          (1, array([[1, 0], [0, 0], [0, 0]], dtype=numpy.float64)),
                          (2, array([[1, 0], [1, 0], [1, 0]], dtype=numpy.float64))])
def test_get_results_non_uniform(daily_bets_percentage_env_non_uniform, current_step, expected_results):
    daily_bets_percentage_env_non_uniform.current_step = current_step
    results = daily_bets_percentage_env_non_uniform.get_results()
    assert numpy.array_equal(results, expected_results)


@pytest.mark.parametrize("current_step,action,expected_reward,finished",
                         [(0, array([0] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([0] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (2, array([0] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 0, True),
                          (0, array([1] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), -1, False),
                          (1, array([1] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 3, False),
                          (2, array([1] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 10, True),
                          (0, array([3] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([3] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 2, False),
                          (2, array([3] * 3 + [0.1] * 6).reshape(3, 3).T.reshape(9), 7, True),
                          (0, array([1, 1] + [0] + [0.1] * 6).reshape(3, 3).T.reshape(9), -1, False),
                          (1, array([1, 2] + [0] + [0.1] * 6).reshape(3, 3).T.reshape(9), 3, False),
                          (2, array([1, 3] + [0] + [0.1] * 6).reshape(3, 3).T.reshape(9), 5, True),
                          (0, array([2, 1] + [0] + [0.1] * 6).reshape(3, 3).T.reshape(9), 1, False),
                          (1, array([2, 2] + [0] + [0.1] * 6).reshape(3, 3).T.reshape(9), -1, False),
                          (2, array([2, 3] + [0] + [0.1] * 6).reshape(3, 3).T.reshape(9), 1, True),
                          (0, array([0] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([0] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (2, array([0] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, True),
                          (0, array([1] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([1] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (2, array([1] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, True),
                          (0, array([3] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([3] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (2, array([3] * 3 + [0] * 6).reshape(3, 3).T.reshape(9), 0, True),
                          (0, array([1, 1] + [0] + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([1, 2] + [0] + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (2, array([1, 3] + [0] + [0] * 6).reshape(3, 3).T.reshape(9), 0, True),
                          (0, array([2, 1] + [0] + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (1, array([2, 2] + [0] + [0] * 6).reshape(3, 3).T.reshape(9), 0, False),
                          (2, array([2, 3] + [0] + [0] * 6).reshape(3, 3).T.reshape(9), 0, True)
                          ])
def test_step_non_uniform(daily_bets_percentage_env_non_uniform, current_step, action, expected_reward, finished):
    daily_bets_percentage_env_non_uniform.current_step = current_step
    odds, reward, done, info = daily_bets_percentage_env_non_uniform.step(action)
    assert reward == expected_reward
    assert done == finished


@pytest.mark.parametrize("current_step,action,expected_reward,finished",  # Initial bank = 10
                         # Bet size = 25% = 2.5$, Reward = -2.5
                         [(0, array([1, 0.25, 0.0]), -2.5, False),
                          # Bet size = [[1.5$, 0.75$]], Reward = 4*1.5 - 1.5 - 0.75 = 3.75
                          (1, array([3, 0.15, 0.075]), 3.75, False),
                          # Bet size = [[2.2$, 2$], [0, 3$], [2.7$, 0]]
                          # Reward = 4*2.2 + 5*2.7 - 2.2 - 2 - 3 - 2.7 = 12.4
                          (2, array([3, 0.22, 0.2, 2, 0, 0.3, 1, 0.27, 0]), 12.4, True),
                          # Validate not using extra odds
                          (0, array([1, 0.25, 0.5]), -2.5, False),
                          # Validate not using extra bets
                          (1, array([3, 0.15, 0.075, 3, 0.2, 0.2]), 3.75, False),
                          # Validate non legal bet
                          (2, array([3, 0.22, 0.2, 2, 0, 0.3, 1, 0.5, 0]), -12.2, False),
                          # Validate legal bet although total percentages exceeds 100
                          (0, array([1, 0.25, 0.8]), -2.5, False),
                          ])
def test_step_non_uniform_non_round_percentage_with_balance(daily_bets_percentage_env_non_uniform, current_step,
                                                            action, expected_reward, finished):
    daily_bets_percentage_env_non_uniform.current_step = current_step
    odds, reward, done, info = daily_bets_percentage_env_non_uniform.step(action)
    assert reward == expected_reward
    assert done == finished
